// Dynamic dialogs 

#include "windows.h"



DLGTEMPLATE* dialogTemplate;
int totalBufferLength;
int usedBufferLength;


    void EnsureSpace(int length)
    {

        if (length + usedBufferLength > totalBufferLength)
        {
        
		void* newBuffer;

            totalBufferLength += length * 2;

            newBuffer = (void *)malloc(totalBufferLength);
            memcpy(newBuffer, dialogTemplate, usedBufferLength);
            
            free(dialogTemplate);
            dialogTemplate = (DLGTEMPLATE*)newBuffer;

        }

    }


    void AppendData(void* data, int dataLength)
    {

        EnsureSpace(dataLength);

        memcpy((char*)dialogTemplate + usedBufferLength, data, dataLength);
        usedBufferLength += dataLength;

    }

    void AppendString(LPCSTR string)
    {

        int length = MultiByteToWideChar(CP_ACP, 0, string, -1, NULL, 0);

        WCHAR* wideString = (WCHAR*)malloc(sizeof(WCHAR) * length);
        MultiByteToWideChar(CP_ACP, 0, string, -1, wideString, length);

        AppendData(wideString, length * sizeof(WCHAR));
        free(wideString);

    }



int DialogTemplate(LPCSTR caption, DWORD style, int x, int y, int w, int h,LPCSTR font, WORD fontSize)
{

        usedBufferLength = sizeof(DLGTEMPLATE );
        totalBufferLength = usedBufferLength;

        dialogTemplate = (DLGTEMPLATE*)malloc(totalBufferLength);

        dialogTemplate->style = style;
        
        if (font != NULL)
        {
            dialogTemplate->style |= DS_SETFONT;
        }
        
        dialogTemplate->x     = x;
        dialogTemplate->y     = y;
        dialogTemplate->cx    = w;
        dialogTemplate->cy    = h;
        dialogTemplate->cdit  = 0;
        
        dialogTemplate->dwExtendedStyle = 0;

        // The dialog box doesn't have a menu or a special class

        AppendData("\0", 2);
        AppendData("\0", 2);

        // Add the dialog's caption to the template

        AppendString(caption);

        if (font != NULL)
        {
            AppendData(&fontSize, sizeof(WORD));
            AppendString(font);
        }
                
	return 0;
    }


    void AlignData(int size)
    {

        int paddingSize = usedBufferLength % size;
        
        if (paddingSize != 0)
        {
            EnsureSpace(paddingSize);
            usedBufferLength += paddingSize;
        }

    }

    void AddStandardComponent(WORD type, LPCSTR caption, DWORD style, DWORD exStyle, int x, int y, int w, int h, WORD id)
    {

        DLGITEMTEMPLATE item;
		WORD preType ;
        // DWORD algin the beginning of the component data
        
        AlignData(sizeof(DWORD));

        item.style = style;
        item.x     = x;
        item.y     = y;
        item.cx    = w;
        item.cy    = h;
        item.id    = id;

        item.dwExtendedStyle = exStyle;

        AppendData(&item, sizeof(DLGITEMTEMPLATE));
        
        preType = 0xFFFF;
        
        AppendData(&preType, sizeof(WORD));
        AppendData(&type, sizeof(WORD));

        AppendString(caption);

        // Increment the component count
        
        dialogTemplate->cdit++;
    
    }


void AddComponent(LPCSTR type, LPCSTR caption, DWORD style, DWORD exStyle,int x, int y, int w, int h, WORD id)
    {

        DLGITEMTEMPLATE item;
		WORD creationDataLength;
        item.style = style;
        item.x     = x;
        item.y     = y;
        item.cx    = w;
        item.cy    = h;
        item.id    = id;

        item.dwExtendedStyle = exStyle;

        AppendData(&item, sizeof(DLGITEMTEMPLATE));
        
        AppendString(type);
        AppendString(caption);

        creationDataLength = 0;
        AppendData(&creationDataLength, sizeof(WORD));

        // Increment the component count
        
        dialogTemplate->cdit++;

    }

    void AddButton(LPCSTR caption, DWORD style, DWORD exStyle, int x, int y,
        int w, int h, WORD id)
    {

        AddStandardComponent(0x0080, caption, style, exStyle, x, y, w, h, id);

        {
		WORD creationDataLength = 0;
        AppendData(&creationDataLength, sizeof(WORD));
		}

    }

    void AddEditBox(LPCSTR caption, DWORD style, DWORD exStyle, int x, int y, int w, int h, WORD id)
    {

        AddStandardComponent(0x0081, caption, style, exStyle, x, y, w, h, id);
    	{
        WORD creationDataLength = 0;
        AppendData(&creationDataLength, sizeof(WORD));
		}
    
    }

    void AddStatic(LPCSTR caption, DWORD style, DWORD exStyle, int x, int y,int w, int h, WORD id)
    {

        AddStandardComponent(0x0082, caption, style, exStyle, x, y, w, h, id);
    
        {
		WORD creationDataLength = 0;
        AppendData(&creationDataLength, sizeof(WORD));
		}
    
    }

    void AddListBox(LPCSTR caption, DWORD style, DWORD exStyle, int x, int y, int w, int h, WORD id)
    {

        AddStandardComponent(0x0083, caption, style, exStyle, x, y, w, h, id);
    
        {
		WORD creationDataLength = 0;
        AppendData(&creationDataLength, sizeof(WORD));
		}
    
    }
    
    void AddScrollBar(LPCSTR caption, DWORD style, DWORD exStyle, int x, int y, int w, int h, WORD id)
    {

        AddStandardComponent(0x0084, caption, style, exStyle, x, y, w, h, id);
    
        {
		WORD creationDataLength = 0;
        AppendData(&creationDataLength, sizeof(WORD));
		}
    
    }

    void AddComboBox(LPCSTR caption, DWORD style, DWORD exStyle, int x, int y,int w, int h, WORD id)
    {

        AddStandardComponent(0x0085, caption, style, exStyle, x, y, w, h, id);
    
        {
		WORD creationDataLength = 0;
        AppendData(&creationDataLength, sizeof(WORD));
		}

    }

/*
    operator const DLGTEMPLATE*() const
    {
        return dialogTemplate;
    }

    virtual ~DialogTemplate()
    {
        free(dialogTemplate);
    }

*/

#ifdef TEST

#define IDEXIT 100


INT_PTR CALLBACK DlgProc(HWND hwnd,UINT msg,WPARAM wparam,LPARAM lparam)
{
switch(msg)
	{
	case WM_INITDIALOG:
		{
		return TRUE;
		}
	case WM_COMMAND:
		{
		switch(LOWORD(wparam))
			{
			case IDEXIT : EndDialog(hwnd,IDEXIT); break;
			}
		} 		
	}
return FALSE;
}


int main(int argc, char* argv[])
{
int res;

DialogTemplate("Test", WS_CAPTION | DS_CENTER,10, 10, 257, 80, "Tahoma",8);

AddStatic("Filename:", WS_VISIBLE, 0, 2 + 3, 7, 37, 8, -1);

AddButton("Exit", WS_VISIBLE, 0,    192 + 3, 62, 56, 13, IDEXIT);

res = DialogBoxIndirect(GetModuleHandle(0), dialogTemplate, NULL, (DLGPROC)DlgProc); 

free(dialogTemplate);


	return 0;
}

#endif
